# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Copyright 2012 Thomi Richards

from gi.repository import Gtk
import os
import os.path
import logging

from graphics import get_pixbuf_for_state
from settings import get_settings

class SettingsDialog(object):
    def __init__(self, jenkins_server_model):
        self.server_model = jenkins_server_model
        self.builder = Gtk.Builder()
        ui_file_path = os.path.join(
            os.path.dirname(
                os.path.realpath(__file__)
                ), 'settings.ui')

        self.builder.add_from_file(ui_file_path)
        self.dlg = self.builder.get_object('settings_dialog')
        self.dlg.connect("response", self._on_response)

        self.url_entry = self.builder.get_object('jenkins_url')
        self.job_model = self.builder.get_object('job_model')
        self.treeview = self.builder.get_object('treeview1')

        self.url_entry.set_text(self.server_model.url)

        cr = Gtk.CellRendererPixbuf()
        cr.set_property("height", 24)
        cr.set_property("width", 24)
        col1 = Gtk.TreeViewColumn("State", cr, pixbuf=0)
        col1.set_resizable(False)
        col1.set_fixed_width(24)
        self.treeview.append_column(col1)

        col2 = Gtk.TreeViewColumn("Job Name", Gtk.CellRendererText(), text=1)
        col2.set_resizable(True)
        self.treeview.append_column(col2)

        cr = Gtk.CellRendererToggle()
        cr.set_property("height", 24)
        cr.set_property("width", 24)
        cr.set_property('activatable', True)
        cr.connect("toggled", self.job_toggled)
        col3 = Gtk.TreeViewColumn("Track Job", cr, active=2)
        col3.set_resizable(False)
        self.treeview.append_column(col3)

        self.builder.connect_signals(self)

        # populate tree view with initial data from server model:
        self.populate_model()
        self.server_model.connect("job-added", self._on_new_job)
        self.server_model.connect("job-removed", self._on_job_removed)

    def populate_model(self):
        jobs = sorted(self.server_model.get_jobs(), key=lambda j: j.name)
        self.job_model.clear()
        for job in jobs:
            self._append_job_to_model(job)
        self.treeview.show()

    def _on_new_job(self, sender, new_job):
        self._append_job_to_model(new_job)
        self.treeview.show()

    def _on_job_removed(self, sender, old_job):
        for i, row in enumerate(self.job_model):
            if row[1] == old_job.name:
                self.job_model.remove(self.job_model.get_iter(i))
                break


    def _append_job_to_model(self, job):
        img = get_pixbuf_for_state(job.color, 24)
        name = job.name
        url = job.url
        monitoring = job.monitored
        self.job_model.append([img, name, monitoring, url])

    def job_toggled(self, cell, path):
        self.job_model[path][2] = not self.job_model[path][2]

    def refresh_jenkins_jobs(self, sender):
        if self.server_model.url != self.url_entry.get_text():
            self.server_model.url = self.url_entry.get_text()
            self.job_model.clear()
        pass
        #TODO - this should trigger the server model to refresh it's data.

    def run(self):
        return self.dlg.run()

    def destroy(self):
        return self.dlg.destroy()

    def _on_response(self, dlg, response_id):
        if response_id == Gtk.ResponseType.OK:
            if self.server_model.url != self.url_entry.get_text():
                self.server_model.url = self.url_entry.get_text()
            for item in self.job_model:
                name = item[1]
                monitor = item[2]
                job = self.server_model.get_job_by_name(name)
                if job.monitored != monitor:
                    job.monitored = monitor
            # update settings:
            get_settings().update_settings_from_model(self.server_model)





